import {SettingCore} from "@core/setting";
import {nextTick} from "vue";

export function getFileType(name) {
    const imgTypes = ["jpg", "jpeg", "png", "gif", "bmp"]
    const excelTypes = ["xlsx", "xls"]
    const wordTypes = ["docx", "doc"]
    const pptTypes = ["ppt", "pptx"]
    const pdfTypes = ["pdf"]
    const exeTypes = ["exe"]
    const zipTypes = ["zip", "rar", "7z"]
    const txtTypes = ["txt"]
    const videoTypes = ["mp4", "avi", "wmv", "flv", "rmvb", "mkv"]
    const audioTypes = ["mp3", "m4a", "wav", "flac", "wma", "aac", "ogg", "ape"]

    const extendName = name.substring(name.lastIndexOf(".") + 1).toLowerCase()
    if (extendName) {
        if (imgTypes.includes(extendName)) {
            return "img"
        } else if (excelTypes.includes(extendName)) {
            return "excel"
        } else if (wordTypes.includes(extendName)) {
            return "word"
        } else if (pptTypes.includes(extendName)) {
            return "ppt"
        } else if (pdfTypes.includes(extendName)) {
            return "pdf"
        } else if (exeTypes.includes(extendName)) {
            return "exe"
        } else if (zipTypes.includes(extendName)) {
            return "zip"
        } else if (txtTypes.includes(extendName)) {
            return "txt"
        } else if (videoTypes.includes(extendName)) {
            return "video"
        } else if (audioTypes.includes(extendName)) {
            return "audio"
        } else {
            return "other"
        }
    } else {
        return "other"
    }
}

export function getFileTypeDefaultImg(name: string): string {
    const fileType: string = getFileType(name);
    let imgUrl: string;
    let imgFile: string;
    const vpath = SettingCore.VITE_TG_VPATH || ''
    switch (fileType) {
        case "img":
            imgFile = 'image-document.png'
            break;
        case "excel":
            imgFile = 'excel-document.png'
            break;
        case "word":
            imgFile = 'word-document.png'
            break;
        case "pdf":
            imgFile = 'pdf-document.png'
            break;
        case "ppt":
            imgFile = 'ppt-document.png'
            break;
        case "exe":
            imgFile = 'exe-document.png'
            break;
        case "zip":
            imgFile = 'zip-document.png'
            break;
        case "txt":
            imgFile = 'txt-document.png'
            break;
        case "video":
            imgFile = 'video-document.png'
            break;
        case "audio":
            imgFile = 'audio-document.png'
            break;
        default:
            imgFile = 'attachment-document.png'
            break;
    }
    if (vpath) {
        imgUrl = `${vpath}/tiangongPublic/images/${imgFile}`
    } else {
        imgUrl = `tiangongPublic/images/${imgFile}`
    }
    return imgUrl
}

export function getFileUrl(file) {
    //获取文件类型默认预览图
    //非数据库读取文件路径
    if (file.saveDb != "N") {
        let subUrl = file.path.replace(/\\/g, "/");
        const path = SettingCore.VITE_BASE_URL + "/files/" + subUrl
        return path
    }
    return ""
}


export function getFileSize(size: number): string {
    if (size < 1024) {
        return size + "B"
    } else if (size < 1024 * 1024) {
        return (size / 1024).toFixed(2) + "KB"
    } else if (size < 1024 * 1024 * 1024) {
        return (size / (1024 * 1024)).toFixed(2) + "MB"
    } else {
        return (size / (1024 * 1024 * 1024)).toFixed(2) + "GB"
    }
}

