/*
*   addApi  添加请求接口
*   editApi  编辑请求接口
*   onSuccess  保存成功后的回调
* */
import {getCurrentInstance, nextTick, reactive, ref, toRefs} from 'vue';
import useFilterRules from './useFilterRules.js'
import {getUUID} from '@coreTools'
import {TgHttp} from "@coreHttp";
import {SettingCore} from "@core/setting";

const {
    addRuleExpression
} = useFilterRules()

// 获取type 类型
const getType = (type = 'add') => {
    let rtn = 'add'
    if (type.constructor === String) {
        rtn = type
    } else if (type.constructor === Object) {
        rtn = type.type || 'add'
    }
    return rtn
}

export default function (param) {

    const {
        addApi = '',
        editApi = '',
        onSuccess,
        tabsSource = [],
        activeTabs = '',
        cascade = {},
        closeDialog = true
    } = param
    const state = reactive({
        form: {},
        span: 12,
        type: '',
        rules: {},
        tabsSource: tabsSource,
        activeTabs: activeTabs,
        cascade: cascade,  // 主表与子表关联字段设置（级联）
        confirmLoading: false,
        UUID: '',
        // show 方法传递的参数存储在如下对象中
        showPassData: ''
    })

    const {proxy} = getCurrentInstance();

    const dialogRef = ref(null)
    const formRef = ref(null)

    // 重置  UUID
    const setUUID = () => {
        state.UUID = getUUID()
    }

    const reset = () => {
        state.confirmLoading = false
        state.form = {}
        // 清空之前的校验
        nextTick(() => {
            // 此处不加延时，表头表体的验证会没有效果 !important
            setTimeout(() => {
                // formRef.value?.clearValidate()
                proxy.$refs.formRef?.clearValidate()
            }, 0)
        })
    }

    const showTabs = (type = 'add', row = {}) => {
        state.type = getType(type)
        let _type = getType(type)

        state.activeTabs = state.tabsSource[0].name
        if (_type === 'add') {
            state.tabsSource.forEach((item, index) => {
                item.disabled = index === 0 ? false : true
            })
        } else {
            state.tabsSource.forEach((item, index) => {
                item.disabled = false
            })
        }
        proxy.$refs.dialogRef?.show()
        nextTick(() => {
            proxy.$refs.EditRef?.show(type, row)
        })
    }

    // 切换tabs
    const changeTabs = (val) => {
        state.activeTabs = val.name

        if (val.name == "first") {
            proxy.$refs.EditRef?.updateHeaderTable && proxy.$refs.EditRef.updateHeaderTable();
        }
        if (val.name != 'first' && val.name != 'close') {
            let parentData = proxy.$refs.EditRef.form
            proxy.$refs[`${state.activeTabs}Ref`]?.tabsSearch && proxy.$refs[`${state.activeTabs}Ref`]?.tabsSearch(parentData)
            // 如下需要注释  TODO
            // proxy.$refs.bodyIndexRef?.tabsSearch(parentData)
        } else if (val.name == 'close') {
            proxy.$refs.dialogRef.close()
        }
    }

    // 关闭弹窗
    const close = () => {
        proxy.$refs.dialogRef?.close()
    }
    /*
    * type
    *   String   add  edit
    *   Object
    *       type String  add edit
    *       data 传输的数据
    * */
    const show = (type = 'add', row = {}, keyValue = '') => {
        // 给触发 重新查询用的参数
        state.showPassData = type
        // 重置 UUID
        setUUID()
        row = JSON.parse(JSON.stringify(row))
        // 兼容表头表体关联类型弹窗，若为表头表体关联，则没有该弹窗

        // dialogRef.value.show()
        proxy.$refs.dialogRef?.show()

        // 重置数据
        reset()
        // 此处 type 赋值的代码需要优化 TODO
        state.type = 'add'
        if (type) {
            const _type = type
            if (_type.constructor === String) {
                if (_type && _type !== 'add') {
                    state.form = row
                }
                state.type = _type
            } else if (type.constructor === Object) {
                const _type = type.type || 'add'
                state.type = _type
                if (_type && _type !== 'add') {
                    state.form = row
                }
            }
        }


        // 表体新增 编辑 时,给与表头的关联的字段的值 进行赋值
        if (keyValue != '') {
            state.form = {
                [cascade.field]: keyValue
            }
        }
    }
    const selectOneById = ({url, data}) => {
        return new Promise(function (resolve, reject) {
            const request = new TgHttp();
            request.entity = data
            request
                .build(SettingCore.VITE_BASE_URL + url)
                .post()
                .then((res) => {
                    if (res.code === 10001) {
                        /*
                        * 新增保存数据以后，type 会变为 edit
                        * */
                        if (state.showPassData.constructor === String) {
                            state.showPassData = state.type
                        } else if (state.showPassData.constructor === Object) {
                            state.showPassData = state.type
                        }

                        resolve({data: state.showPassData, row: res.data})
                    }
                })
        })
    }
    const save = () => {
        // 去除按钮 loading
        state.confirmLoading = true

        // formRef.value.validate((valid) => {
        proxy.$refs.formRef.validate((valid) => {
            if (valid) {
                const request = new TgHttp();
                request.entity = state.form
                request
                    .build(state.type === 'add' ? SettingCore.VITE_BASE_URL + addApi : SettingCore.VITE_BASE_URL + editApi, {noMsg: true})
                    .post()
                    .then((data) => {
                        if (data.code === 10001) {
                            // dialogRef.value.close()
                            !!closeDialog && proxy.$refs.dialogRef?.close()
                            tg.msg.success(data.msg)
                            /*
                            * 保存成功以后的回调
                            * 比如触发页面查询条件
                            * */
                            onSuccess && onSuccess(data)

                            // 多表体时，需要返回值，后面表体需要使用  此处存在隐患，如果返回值被设置了其他值
                            state.form = data.data
                            state.type === 'add' ? state.type = 'edit' : ''
                        } else {
                            tg.msg.warning(data.msg)
                        }
                        // 去除按钮 loading
                        setTimeout(() => {
                            state.confirmLoading = false
                        }, 500)

                        // 重置 UUID
                        setUUID()
                    }).catch(e => {
                    // 去除按钮 loading
                    setTimeout(() => {
                        state.confirmLoading = false
                    }, 500)
                    // 重置 UUID
                    setUUID()
                })

                // TgRequest({
                //     headers: {
                //         // 浏览器默认将 UUID  转成  Uuid
                //         Uuid: state.UUID
                //     }
                // })
            } else {
                state.confirmLoading = false
            }
        })
    }

    // 表体保存
    const tabsSave = (val) => {
        // 合并字段
        state.form = Object.assign({}, state.form, {
            // [state.cascade.headField]:val
            [state.cascade.field]: val
        })
        save()
    }

    // 更新tabs数据(表头保存成功后更新tabs)
    const updateTabs = () => {
        state.tabsSource.forEach((item, index) => {
            item.disabled = false
        })
    }

    /*
    *   校验
    *       addRuleEx('gNum,itemNo')
    * */
    const addRuleEx = (codeList, extend) => {
        state.rules = addRuleExpression(codeList, extend)
        /*
        *   此处不加延时，表头表体的验证会没有效果 !important
        *   此处加清空表单校验：目的是当校验规则为异步时
        * */
        setTimeout(() => {
            // formRef.value?.clearValidate()
            proxy.$refs.formRef?.clearValidate()
        }, 0)
    }
    return {
        ...toRefs(state),

        save,
        show,
        addRuleEx,
        showTabs,
        close,
        changeTabs,
        tabsSave,
        updateTabs,
        selectOneById,
        state
    }
}
