import {getCurrentInstance, reactive, ref, toRefs} from 'vue';
import {ElMessageBox} from 'element-plus';
import {optionConfig} from './useConfig'
import {handleClearFilterQuery, handleFilterQuery, handleFormData, initQuery} from './searchPackgeData'
import {TgHttp} from "@coreHttp";
import {SettingCore} from "@core/setting";
import TgRequest from "@coreRequest";
import {tmn, tm, t} from '@i18nHelper'
// 初始化查询条件
const Page = {
    sortField: '',
    asc: '',
    // 排序
    sort(pagination, sortField) {
        let rtn = {}
        if (!!pagination && pagination.sortField) {
            rtn.asc = pagination.asc
            rtn.sortField = pagination.sortField
            Page.asc = pagination.asc
            Page.sortField = pagination.sortField
        } else {
            if (Page.sortField && Page.sortField !== '' && !(pagination && JSON.stringify(pagination) === '{}')) {
                rtn.asc = Page.asc
                rtn.sortField = Page.sortField
            } else {
                rtn.sortField = sortField
                rtn.asc = false
                Page.asc = ''
                Page.sortField = ''
            }
        }
        return rtn
    }
}

/*
* param 传入的参数
*   listApi 查询列表的 api 接口
*   sortField   排序字段
*   searchApi   请求的地址
*   deleteApi  删除的地址
*   queryField
*       type: l =
*       table:
* */
export default function (param = {}) {
    let {
        searchApi,
        deleteApi,
        sortField = '',
        queryField = {},
        cascade,
        keyValue,
        keyField,
        version = '',
        setting,
    } = param

    // 编辑弹框
    const {proxy} = getCurrentInstance();
    const EditRef = ref('')

    const state = reactive({
        headData: {},
        form: {},
        queryParams: initQuery(),      // 查询条件
        total: 0,
        tableData: [],
        listLoading: false,
        selectionList: [],  // 选中的值
        queryField: queryField,     // 过滤条件类别设置，不传值则默认 l (like)
        cascade: cascade,  // 主表与子表关联字段设置
        keyValue: keyValue,  // 主表与子表关联字段对应的值
        keyField: keyField,// 关键字段 删除时用到的
        version: version,  //   用于私有化配置: 筛选配置，表格配置
        // 私有化配置项
        setting: setting
    })

    // 初始化查询条件
    // state.queryField = queryField

    // 组装发送的数据
    const _PackageSendData = ({pagination = {}}) => {
        // 排序
        state.queryParams = Object.assign({}, state.queryParams, Page.sort(pagination, sortField))// 排序

        !pagination && (state.queryParams.pageNum = 1)  // 如果不存在分页操作数据，则需要重置分页数据

        let queryFieldStr = handleFilterQuery(state.queryField)  // 筛选条件
        // 过滤多余的筛选条件匹配规则
        queryFieldStr = handleClearFilterQuery(queryFieldStr, state.form)

        // 分页条件
        state.queryParams.pageNum = pagination && pagination.page && (typeof pagination.page === 'number') ? pagination.page : state.queryParams.pageNum
        state.queryParams.pageSize = pagination && pagination.limit && (typeof pagination.limit === 'number') ? pagination.limit : state.queryParams.pageSize;


        // 查询的请求参数
        return Object.assign({},
            {sortField: sortField || ''},      // 排序字段
            handleFormData(state.form, queryFieldStr),
            state.queryParams,
            {queryField: queryFieldStr}
        )
    }

    // 查询
    const pagination = (pagination, type = 'search') => {

        let sendData = _PackageSendData({pagination})

        state.listLoading = true

        // const request = new TgHttp();
        // request.entity = sendData
        // request
        //   .build(SettingCore.VITE_BASE_URL + searchApi)
        //   .post()
        TgRequest({
            url: searchApi,
            method: 'post',
            data: sendData
        })
            .then(data => {
                state.listLoading = false
                state.tableData = data?.data?.list ?? []
                state.total = data?.data?.total ?? 0;
            }).catch(e => {
            state.listLoading = false
        })
    }

    // 表体数据组装
    const _PackageTabsSearch = (row) => {
        // 组装查询条件(在页面内部组装好了,这里不用了)
        // state.queryField=state.cascade.field+'|='
        state.form = {
            ...state.form,
            // [state.cascade.headField]:row[state.cascade.field]
            [state.cascade.field]: row[state.cascade.headField]
        }
        // state.keyValue=row[state.cascade.field]
        state.keyValue = row[state.cascade.headField]

        if (proxy.$refs.EditRef) {
            proxy.$refs.EditRef.type = 'add'
            // 给与表头关联的表体的字段 赋值
            proxy.$refs.EditRef.form = {
                // [state.cascade.headField]:row[state.cascade.field]
                [state.cascade.field]: row[state.cascade.headField]
            }
        }
    }

    // 表体查询
    const tabsSearch = (row = {}, type = 'search', param = {}) => {
        state.headData = row
        const {before = null} = param
        _PackageTabsSearch(row)
        // 用于业务扩展：比如查询设置业务初始化字段
        before && before()

        search(type)
        // 初始化编辑页: 新增
        // proxy.$refs.EditRef?.show('add')
    }
    // 查询按钮
    const search = (type = 'search') => {
        state.queryParams.pageNum = 1
        pagination('', type)
    }

    // 查询： TgFilter
    const TgSearch = (data = {}) => {
        // 如果是表体的查询，先设置
        if (state.headData && JSON.stringify(state.headData) !== '{}') {
            _PackageTabsSearch(state.headData)
        }
        state.form = Object.assign({}, state.form, data)
        search()
    }

    // 清空
    const reset = () => {
        state.form = {}
        state.queryParams = initQuery()      // 防止之前的分页影响后面的数据展示
    }

    // 重置： TgFilter
    const TgReset = () => {
        reset()
    }


    /*
    *    新增 编辑
    *       type:  edit  add
    *       keyValue: 表体新增 编辑 时,给与表头的关联的字段的值 进行赋值
    * */
    const edit = (type, row, keyValue) => {
        // editDialogRef.value.show(type,row)
        proxy.$refs.EditRef.show(type, row, keyValue)
    }

    /*
    * 删除
    *   list 单个删除的数据
    *   paramDelete
    *       key: 删除的 标志
    *       del([scope.row], { key: 'ids' })
    * */
    const del = (list, param) => {

        const {key = 'ids', keyField = 'id', beforeSearch, requestErrorAfter} = param
        const _search = param.search

        return new Promise(function (resolve, reject) {
            ElMessageBox.confirm(
                t('tg.page.msg.M00132'),
                t('tg.page.common.prompt'),
                {
                    confirmButtonText: t('action.sure'),
                    cancelButtonText: t('action.cancel'),
                    type: 'warning',
                }
            ).then(() => {
                // 批量删除
                let temp = []
                for (let p in list) {
                    // temp.push(list[p]['id'])
                    temp.push(list[p][keyField])
                }

                // 发送的数据
                let sendData = {}
                sendData[key] = temp.join(',')

                const request = new TgHttp();
                request.entity = sendData
                request
                    .build(SettingCore.VITE_BASE_URL + deleteApi)
                    .post()
                    .then(data => {
                        if (data.code === 10001) {
                            tg.msg.success(t('tg.page.tgPage.queryPage.M00057'))

                            beforeSearch && beforeSearch()
                            // 查询
                            if (_search) {
                                _search()
                            } else {
                                search();
                            }

                            resolve(data)
                        } else {
                            requestErrorAfter && requestErrorAfter()
                        }
                    })
            })
        })
    }

    // 自定义筛选配置和表格配置数据
    const customConfig = () => {
        const request = new TgHttp();
        request.entity = {tableCode: version}
        let url = SettingCore.VITE_BASE_URL + "/sysTableSetting/selectUserSetting"
        if (window.website.tgApiPrefix) {
            url = SettingCore.VITE_BASE_URL + "/" + window.website.tgApiPrefix + "/sysTableSetting/selectUserSetting";
        }
        request
            .build(url)
            .post()
            .then(data => {
                if (data.code === 10001) {
                    state.setting = optionConfig(setting, data.data)
                }
            })
    }

    /*
    *   表头、单表导出
    *   exportApi  导出的地址
    *   downloadExcelApi    下载Excel的地址
    *   fileName 下载的文件名
    *   当传入 table 字段： exportApi，downloadExcelApi 可以不传
    * */
    const exportExcel = (_param) => {
        let {table = '', exportApi, downloadExcelApi, fileName, data = {}} = _param
        if (state.tableData.length === 0) {
            tg.msg.warning(t('tg.page.msg.M00134'))
            return
        }
        if (table && table !== '') {
            let str = ''
            str = import.meta.env.VITE_BASE_URL
            // str = 'http://192.168.106.185:10002'
            exportApi = `${str}/${table}/export`
            downloadExcelApi = `${str}/${table}/downloadExcel`
        }
        // search('export')
        state.queryParams.pageNum = 1

        // 组装发送的数据
        let sendData = _PackageSendData({pagination: {}, downloadExcelApi})

        // if(type==='export'){
        let tempExcel = []

        let columnField = state.setting.selectList
        columnField.forEach((item, index) => {
            if (item.export && item.export + '' !== '1') {
                let temp = {
                    columnCode: item.columnCode,
                    // columnName:item.label?(proxy.$t(item.label)?proxy.$t(item.label):item.columnCode):item.columnCode
                    columnName: item.labelName || proxy.$t(item.label)
                }
                tempExcel.push(temp)
            }
        })
        sendData.excel = tempExcel
        // console.log(44,state.columnField, sendData.excel)
        // }

        const request = new TgHttp();
        request.entity = {
            ...sendData,
            ...data
        }
        let url = SettingCore.VITE_BASE_URL + exportApi
        if (exportApi.indexOf('http') >= 0) {
            url = exportApi
        }
        request
            .build(url)
            .post()
            .then(_data => {
                if (_data && _data.code === 10001) {
                    downloadExcel({
                        id: _data?.data || '',
                        downloadExcelApi,
                        fileName
                    })
                }
            })
    }

    // 表体导出
    const exportItemExcel = (param) => {
        // tabsSearch('','export')
        // _PackageTabsSearch('')
        exportExcel(param)
    }

    // 下载
    /*
    *     downLoadFileName:downLoadFileName,// 下载的文件名
    * */
    const downloadExcel = ({id, downloadExcelApi, fileName}) => {
        // todo
        TgRequest({
            url: downloadExcelApi + '?id=' + id,
            method: 'get',
            responseType: 'blob',// Excel导出，若后台返回是经过处理的文件流数据，则get请求时一定要加该设置，否则下载的文件打不开
        })
            .then((blob) => {
                let href = window.URL.createObjectURL(new Blob([blob]));
                let link = document.createElement('a');
                link.style.display = 'none';
                link.href = href;
                link.setAttribute('download', fileName + '.xlsx');
                document.body.appendChild(link);
                link.click();
                document.body.removeChild(link); // 下载完成移除元素
                window.URL.revokeObjectURL(href); // 释放掉blob对象
            })
    }

    // 表格切换选择
    const selectionChange = (val) => {
        state.selectionList = val || []
    }


    return {
        ...toRefs(state),
        // 用于 TgTableList
        paginationData: () => {
            return {
                total: state.total,
                page: state.queryParams.pageNum,
                limit: state.queryParams.pageSize
            }
        },
        search: TgSearch,  // 查询
        pagination, // 翻页
        // reset,  // 清空
        reset: TgReset,  // 清空
        edit,     // 新增编辑: 共用
        del,   // 删除
        // HandleuserDefined,       // 用户自定义事件
        tabsSearch,     //
        exportExcel,    // 导出Excel
        exportItemExcel,// 表体导出
        // 自定义筛选配置和表格配置数据
        customConfig,
        selectionChange
    }
}
