import {createI18n} from 'vue-i18n'
import getLanguage from '@core/utils/TgLanguage'
import TgLocalStorage from "@core/utils/TgLocalStorage";

export let i18n ;

const message = ((modules) => {
    let obj = {}
    for (const key in modules) {
        obj = {
            ...obj,
            ...modules[key].default
        }
    }
    return obj
})

async function createI18nOptions() {
    const locale = TgLocalStorage.get('lang') || 'ch';

    let defaultLocal = import.meta.globEager('../../language/ch/**/*.js');
    const localEn = import.meta.globEager('../../language/en/**/*.js');
    // let defaultLocal = import.meta.globEager('@core/language/ch/**/*.js');
    // const localEn = import.meta.globEager('@core/language/en/**/*.js');

    let defaultObj = message(defaultLocal)
    try{
        let defaultLocalSelf = import.meta.globEager('../../../web/app/language/ch/**/*.js');
        let defaultLocalObj = message(defaultLocalSelf)
        defaultObj = Object.assign({},defaultObj,defaultLocalObj)
    }catch(e){ }

    return {
        fallbackLocale: 'ch',
        globalInjection: true,
        legacy: false, // you must specify 'legacy: false' option
        // locale: 'en',
        locale: locale,
        messages: {
            // [locale]: message(defaultLocal)
            ch: defaultObj,
            en: message(localEn),
        },
        silentTranslationWarn: true // 关闭vue-i18n warnning提示
    };
}
export async function setupI18n(app) {
    const options = await createI18nOptions();
    app.config.globalProperties.getLanguage = getLanguage
    i18n = createI18n(options);
    app.use(i18n);
}
