import BasePage from '../BasePage'
import {getCurrentInstance, nextTick, onMounted, reactive, toRefs} from "vue";
import tg from '@coreTgPlus'
import pageUtils from "@/tiangongCore/utils/common/pageUtils";
import $ from "jquery"
import {TgHttp} from "@coreHttp";
import {t, tm} from '@i18nHelper'

export default function () {
    const page = {}
    const basePage = BasePage(page)
    const data = reactive({
        ...toRefs(basePage.data),
        pageInfo: {type: "EditPage", showed: false},//页面模型信息
        isAdd: true,
        editConfig: {saveAdd: false, saveClose: false, saveLock: false},//编辑的配置信息
        options: {},
        eciFiles: [],//子页面信息
        saveApi: "",//保存api
        loadApi: "",//编辑api
        saveType: "",//保存类型，跟后台配合
        isDialog: false,
        mode: "",//edit编辑需要的参数
        formProps: {},//表单属性对象
        saveSuccessMessage: tm("tg.page.tgPage.editPage.M00070", "保存成功"),//保存成功的提示信息
    })

    const {proxy} = getCurrentInstance();

    page.data = data
    page.data.pageInfo.type = 'EditPage'

    onMounted(() => {
        // page.autoEntity("editForm")

        //回车
        enterTab();
    })

    /**
     * 表单回车
     */
    function enterTab() {
        nextTick(() => {
            nextTick(() => {
                if (page.context.getRef("editForm")) {
                    //如果界面使用了tg-form，就直接给调用tgform里面的setAutoFocus
                    let editForm = page.context.getRef("editForm")
                    if (editForm && editForm.setAutoFocus) {
                        editForm.setAutoFocus()
                    } else {
                        //bug:展开或者收缩的框，展开以后，会焦点默认到第一个，因为这边设置了true
                        let formEl = editForm.$el
                        //在edit界面中使用tg-form的时候，就需要找到nextElementSibling元素
                        if (formEl.nextElementSibling) {
                            formEl = formEl.nextElementSibling
                        }
                        tg.enterTab(formEl, true)
                    }

                }
            })
        })
    }

    /**
     * 针对是没有List界面打开的Edit界面，调用这个方法，初始化界面的一些信息
     * @param row
     * @param state
     */
    page.useEditPage = (row, state) => {
        page.data.isAdd = !row;
        if (row) {
            page.data.entity = pageUtils.convertToUIEntity(page, row);
        } else {
            page.add(state)
        }
    }

    page.load = async function (state) {

        if (!page.data.loadApi) {
            tg.msg.error(t("tg.page.tgPage.editPage.M00078", [page.context.name]));//页面{0}没有设置loadApi
            return;
        }

        page.setFormloading(true);

        let request = new TgHttp()
        request.entity[page.data.keyField] = page.data.keyValue;

        if (page.loading) {
            page.loading(request)
        }

        let response = await request.build(page.data.loadApi).post()
        if (response?.success) {

            let entity = response.data
            if (!entity) {
                page.setFormloading(false);
                tg.msg.error(t("tg.page.tgPage.editPage.M00076")); //数据加载异常,没有查询到对应的数据
                return;
            }

            if (page.binding) {
                page.binding(response, state);
            }

            page.data.entity = pageUtils.convertToUIEntity(page, entity);

            page.createHeadRow();

            if (state == undefined) {
                state = {};
            }

            if (state.byCopy) {
                page.data.isAdd = true;
                page.data.entity[page.data.keyField] = "";
                page.unlockAction();
            }

            if (page.loaded) {
                page.loaded(response, state);
            }

            page.xtLoaded(response, state);

            // page.data.pageInfo.type
            if (page.context.getContextPage() && page.context.getContextPage().data.pageInfo.type == "EditTab") {
                page.context.getContextPage().headChanged(page.data.entity)
            }
        }
        page.setFormloading(false);

    }

    page.createHeadRow = function () {
        if (page.data.cascade) {
            let field = page.data.cascade.field;
            let headField = page.data.cascade.headField;

            if (!field) {
                throw Exception(t("tg.page.tgPage.editPage.M00066"))//编辑页面:级联配置不正确
            }

            if (!headField) {
                throw Exception(t("tg.page.tgPage.editPage.M00066"))//编辑页面:级联配置不正确
            }

            if (!page.data.headRow) {
                page.data.headRow = {};
            }

            let cascadeValue = page.data.entity[field];

            if (!cascadeValue) {
                throw Exception(t("tg.page.tgPage.editPage.M00077")) //编辑页面:级联值不可以为空
            }

            page.data.headRow[headField] = cascadeValue;

        }
    }
    /**
     * 协同处理子页面的数据
     */
    page.xtLoaded = function (response, state) {
        let row = response.entity;
        for (let index in page.context?.children) {
            let child = page.context?.children[index]
            if (child.sub) {
                //作为编辑页面的子表的列表页
                child.setHeadRow(row)

                if (!state.headOnly) {
                    row && child.search && child.search()
                }
            } else if (child.editPart) {
                //编辑部件显示
                child.show(row)
            }
        }
    }

    /**
     * 保存
     */
    page.save = function (state) {

        if (!page.data.saveApi) {
            tg.msg.error(t("tg.page.tgPage.editPage.M00079", [page.context.name]));//页面{0}没有设置saveApi
            return;
        }
        state = state || {}

        if (state.currentTarget) {
            state = undefined;
        }

        state = state || {}

        let type = state.type;
        let addon = state.addon;

        for (let x in page.data.entity) {
            if (page.data.entity[x]) {
                if (typeof (page.data.entity[x]) == "string") {
                    page.data.entity[x] = page.data.entity[x].trim();
                }
            }
        }

        let form = page.context.getRef("editForm");
        if (!form) {
            throw Exception(t("tg.page.tgPage.editPage.M00067"))  //当前界面没有编辑页面
        }

        form.validate(async (valid) => {
            if (valid) {
                let request = new TgHttp()
                request.entity = tg.copy(page.data.entity);
                request.entity = pageUtils.convertToAPIEntity(page, request.entity, false);

                //隐藏表单
                for (let item in page.data.formData) {
                    request.entity[item] = page.data.formData[item];
                }

                if (addon) {
                    for (let item in addon) {
                        request.entity[item] = addon[item];
                    }
                }

                if (page.data.saveType) {
                    request.type = page.data.saveType
                }

                if (type) {
                    request.type = type;
                }

                if (page.data.cascade) {
                    let field = page.data.cascade.field;
                    let headField = page.data.cascade.headField;

                    if (!field) {
                        throw Exception(t("tg.page.tgPage.editPage.M00066"))//编辑页面:级联配置不正确
                    }

                    if (!headField) {
                        throw Exception(t("tg.page.tgPage.editPage.M00066"))//编辑页面:级联配置不正确
                    }

                    if (!page.data.headRow) {
                        throw Exception(t("tg.page.tgPage.editPage.M00068"))//编辑页面:配置了级联，但是headRow为空
                        //编辑页面:配置了级联，但是headRow为空
                    }

                    let cascadeValue = page.data.headRow[headField];

                    if (!cascadeValue) {
                        //编辑页面:配置了级联,但是级联值为空
                        throw Exception(t("tg.page.tgPage.editPage.M00069"))
                    }

                    request.entity[field] = cascadeValue;
                    request.entity["$cascadeField"] = field;

                }
                if (page.data.isAdd) {
                    request.businessType = "INSERT"
                } else {
                    request.businessType = "UPDATE"
                }

                request.isAdd = page.data.isAdd;

                if (page.saving) {
                    await page.saving(request, state);
                }

                if (request.cancel) {
                    return;
                }

                page.setFormloading(true);
                request["tgTime"] = Date.parse(new Date())
                let response = await request.build(page.data.saveApi).post()

                if (response?.success) {
                    let entity = response.data
                    page.data.entity = pageUtils.convertToUIEntity(page, entity);

                    page.saveSuccess(response, request);

                    if (page.saved) {
                        page.saved(response, state)
                    }

                    page.xtSaved(response, state);

                } else {
                    page.setFormloading(false)
                }

            } else {

                $(".el-form-item__error").click(function () {
                    page.clearValidate(true);
                });

            }
        })
    }

    /**
     * save成功
     * @param response
     * @param request
     */
    page.saveSuccess = function (response, request) {

        page.data.dataChanged = true;
        page.data.isAdd = false;
        page.data.keyValue = page.data.entity[page.data.keyField];

        page.setFormloading(false);

        page.setTitle();

        if (page.context.parent == undefined) {
            page.context.parent = page.context.getContextPage();
        }
        if (page.context.getContextPage()) {
            if (!page.context.getContextPage().data.pageInfo) {
                return;
            }
        }

        let state = {dialog: page.data.isDialog}
        //通知Tab页面用于控制子页面tabLock=false 子页面的headRow赋值问题
        //树页面保存，通知树页面修改树节点
        //通知EditBox

        if (page.context.getContextPage()) {
            !page.root && page.context.getContextPage().headSave && page.context.getContextPage().headSave(page.data.entity, request.isAdd, state);
        }

        if (page.data.editConfig.saveAdd) {
            page.add();
        }

        if (page.data.editConfig.saveLock) {
            page.lockAction();
        }

        if (page.data.editConfig.saveClose) {
            page.data.visible = false
            if (page.context.getContextPage().endEdit) {
                page.context.getContextPage().endEdit();
            }
        }

        //保存成功
        //保存成功的提示信息，业务系统想自定义，所以定义一个变量。
        //2024.5.21 丁志鹄
        tg.msg.success(page.data.saveSuccessMessage)

    }
    /**
     * save以后，协同处理
     * @param response
     * @param state
     */
    page.xtSaved = function (response, state) {
        let row = page.data.entity;
        for (let index in page.context.children) {
            let child = page.context.children[index]
            if (child.sub) {
                //作为编辑页面的子表的列表页
                child.setHeadRow(row)
            }
        }
    }
    /**
     * save报错以后
     */
    page.saveError = function (response, request) {
        page.data.formLoading = false;
    }

    /**
     * 编辑
     */
    page.edit = function (row, state = {}) {

        if (page.data.mode != "none") {
            page.data.isAdd = false

            if (state.lockEdit) {
                page.lockAction()
            }

            //编辑的时候，页面还没有初始化

            if (!page.data.keyField) {
                // "数据加载异常,页面没有配置keyField"
                tg.msg.error(t("tg.page.tgPage.editPage.M00072"));
                return;
            }

            page.setTitle();

            page.data.keyValue = row[page.data.keyField];

            if (!page.data.keyValue) {
                //数据加载异常,修改数据时主键值不可以为空
                tg.msg.error(t("tg.page.tgPage.editPage.M00071"));
                return;
            }

            page.load(state);
        }
    }
    /**
     * 新增
     * @param state
     */
    page.add = function (state) {

        state = state || {}

        if (state.isTrusted && page.data.isAdd) {
            //当前页面数据还没有保存,是否继续新增?
            tg.msg.confirm(t("tg.page.tgPage.editPage.M00073") + "?").then(() => {
                page.addBase(state);
            })
        } else {
            page.addBase(state);
        }
    }
    page.addBase = function (state) {
        state = state || {}

        if (page.adding) {
            page.adding(state);
        }

        page.data.isAdd = true

        page.data.keyValue = "";

        page.unlockAction();

        page.setTitle();

        if (state == undefined) {
            state = {};
        }

        if (!state.byShow) {

            autoFocus();
        }

        page.resetEntity();

        //自动处理 cascade headRow
        //修改entity的关联值，如果为空的情况
        if (page.data.isAdd && page.data.cascade && page.data.headRow) {
            let value = page.data.headRow[page.data.cascade.headField];
            page.data.entity[page.data.cascade.field] = value;

            let map = page.data.cascade.map
            if (map) {
                for (let key in map) {
                    page.data.entity[key] = page.data.headRow[map[key]]
                }

                if (page.data.coreUI) {
                    page.data.entity = tg.copy(page.data.entity);
                }
            }
        }

        !page.root && page.context.getContextPage() && page.context.getContextPage().headAdd && page.context.getContextPage().headAdd(state);

        autoFocus();

        // tg.firstFocus();

        if (page.added) {
            page.added(state)
        }

        page.xtAdded(state);

        nextTick(() => {
            nextTick(() => {
                page.clearValidate();
            })
        })
    }
    page.xtAdded = function (state) {
        nextTick(() => {
            for (let index in page.context?.children) {
                let child = page.context?.children[index]
                if (child?.sub) {
                    //作为编辑页面的子表的列表页
                    child.setHeadRow(null)
                    if (child.context.getRef("table")) {
                        child.clearTable();
                    }
                }
            }
        })
    }
    page.clearValidate = function (needRecover) {

        if (page.context.getRef("editForm")) {
            let entityCheck = page.data.entityCheck;
            let editFormCheck = page.context.getRef("editForm").entityCheck
            page.context.getRef("editForm").entityCheck = {} //清除放大镜的验证
            page.context.getRef("editForm").clearValidate() // 只清除清除验证

            page.data.entityCheck = {}

            if (needRecover) {
                setTimeout(() => {
                    page.data.entityCheck = entityCheck
                }, 10);
            }

            setTimeout(() => {
                if (page.context.getRef("editForm")) {
                    page.context.getRef("editForm").entityCheck = {} //清除放大镜的验证
                    page.context.getRef("editForm").clearValidate() // 只清除清除验证
                }
            }, 100);
        }
    }

    function autoFocus() {
        if (window.$eciFocusInput) {
            let $input = $(window.$eciFocusInput)
            $input.focus()
        }
    }

    page.lockAction = function () {
        page.data.lockFlag = true
    }
    page.unlockAction = function () {
        page.data.lockFlag = false
    }

    page.setTitle = function () {

        if (page.data.bizName == undefined || page.data.bizName == "") {
            return;
        }
        let defaultName = t("tg.page.tgPage.editPage.M00074");   //数据
        let addName = t("action.add");// "新增";
        let editName = t("action.edit");//"编辑";
        let viewName = t("tg.page.tgPage.editPage.view");   //查看
        let titlePart = addName;
        if (!page.data.isAdd) {
            if (page.data.isView) {
                titlePart = viewName;
            } else {
                titlePart = editName;
            }
        }
        page.data.title = (page.data.bizName ? page.data.bizName : defaultName) + "-" + titlePart; // (this.isAdd ? addName : editName);

        let contextPage = page.context.getContextPage()
        if (contextPage && contextPage.isBasePage && contextPage.data.pageInfo.type == "EditBox") {
            contextPage.title = page.data.title;
        }
    }

    page.setFormloading = function (flag) {
        page.data.formLoading = flag;
    }

    page.useEditForm = function (props) {
        //设置form表单的属性
        page.data.formProps = props
    }

    page.resetEntity = function () {

        for (let item in page.data.entity) {

            let config = page.data.entityConfig[item];

            let defaultValue = "";

            if (page.data.entityDefault[item]) {
                defaultValue = page.data.entityDefault[item];

                if (defaultValue == "#yyyy-MM-dd#") {
                    defaultValue = tg.now().format("yyyy-MM-dd");
                } else if (defaultValue == "#yyyy-MM-dd HH:mm:ss#") {
                    defaultValue = tg.now().format("-ss");
                } else if (defaultValue == "{-ss}") {
                    defaultValue = tg.now().format("-ss");
                } else if (defaultValue == "{-dd}") {
                    defaultValue = tg.now().format("-dd");
                }
            } else if (page.data.entityDefault[item] != "" && page.data.entityDefault[item] == 0) {
                defaultValue = 0;
            }

            if (config) {
                if (config.keep) {
                    continue;
                }
                if (config.newid) {
                    page.data.entity[item] = tg.guid32();
                    continue;
                }
                if (config.default) {
                    page.data.entity[item] = config.default;
                    continue;
                } else if (config.default == 0) {
                    page.data.entity[item] = config.default;
                    continue;
                }

                if (config.isArray) {
                    if (defaultValue != undefined && defaultValue != null) {
                        page.data.entity[item] = defaultValue
                    } else {
                        page.data.entity[item] = []
                    }
                } else {
                    page.data.entity[item] = defaultValue;
                }

                if (config.eciFile) {
                    page.data.entity[item] = tg.guid();
                }

            } else {
                page.data.entity[item] = defaultValue;
            }
        }

        //需要通过配置来设置entity
        for (let prop in page.data.entityConfig) {
            let item = page.data.entityConfig[prop];
            if (item.eciFile) {
                page.data.entity[prop] = tg.guid();
            }
            if (item.newid) {
                page.data.entity[prop] = tg.guid32();
            }
        }
    }

    page.httpx = function (api, options) {
        options = tg.extend(options);

        let request = page.http(api, options);

        if (!options.mode) {
            options.mode = 'edit';
        }

        if (options.mode == "edit") {
            request[page.data.keyField] = page.data.entity[page.data.keyField];
        }

        if (options.mode == "entity") {
            request.entity = page.data.entity;
        }

        return request;
    }

    page.pageClear = function () {
        let contextPage = page.context.getContextPage()

        if (!page.root && contextPage && contextPage.isBasePage && contextPage.data.pageInfo.type == "EditTab") {
            contextPage.row = null;
        }

        page.data.isAdd = true;
        page.resetEntity();
        page.eciFileClear();
        page.basePageClear();
        page.clearValidate();

        page.data.pageInfo.showed = false;

        if (page.pageChoose) {
            for (let index in page.pageChoose) {
                page.pageChoose[index].isError = false;
            }
        }
    }
    page.eciFileClear = function () {
        if (page.context.children) {
            for (let index in page.context.children) {
                let child = page.context.children[index]
                child.pageClear()
            }
        }
    }

    page.show = function (row, state) {
        page.baseShow(row, state)
    }
    page.showAdd = function (state) {
        page.baseShow(null, state)
    }
    page.baseShow = function (row, state) {
        page.data.visible = true
        if (page.pagePopType.popType != 'page') {
            const ret = page.dialogOpen()
            //如果有最小化的dialog，就直接打开了
            if (ret == false) {
                return
            }

        }
        page.context.parent = page.context.getContextPage()

        state = tg.extend(state)

        state.byShow = true;

        page.setView(state);

        page.data.dataChanged = false;

        if (row) {
            page.edit(row, state);
        } else {
            page.add(state);
        }

        // if (this.mode != "none") {
        //     this.setTitle();
        // }

        nextTick(() => {
            nextTick(() => {
                page.autoEntity("editForm");
                enterTab();
                if (page.showed) {
                    page.showed(row, state)
                }
            })
        })

        page.clearValidate();

        page.data.pageInfo.showed = true;

    }

    /**增加下级 配合TreePage使用*/
    page.addNext = function () {
        page.context.getContextPage().addNext();
    }

    /**增加同级 配合TreePage使用*/
    page.addSame = function () {
        page.context.getContextPage().addSame({byEditPage: true});
    }

    page.validateField = function (prop) {
        page.context.getRef("editForm").validateField(prop)
    }

    return {
        page
    }
}
