import BasePage from "../BasePage";
import {getCurrentInstance, onBeforeMount, reactive, toRefs,} from "vue";
import tg from "@coreTgPlus";
import {TgHttp} from "@coreHttp";
import {t} from '@i18nHelper'

export default function () {
    const page = {};
    const basePage = BasePage(page);
    const data = reactive({
        ...toRefs(basePage.data),
        pageInfo: {type: "TreePage"},
        defaultProps: {
            children: "children",
            label: "label",
            isLeaf: 'isLeaf'
        },
        rootNode: {
            id: "",
            label: "",
            parentId: "",
        },
        editConfig: {click: true},
        checkedKeys: [],
        hasCopy: false,
        showMenu: false,
        clickCount: 0,
        editNode: null,
        parentNode: null,
        currentNode: null,
        copyNode: null,
        treeConfig: {clickExpand: true},
        entity: {searchData: ""},
        treeData: [],
        filterText: "",
        nameField: "",
    });

    const {proxy} = getCurrentInstance();

    page.data = data;
    page.data.pageInfo.type = "TreePage";

    onBeforeMount(() => {
        if (!data.rootNode.label) {
            page.getRoot()
        }
    })
    page.getRoot = () => {
        page.getRootBase()
    }
    //获取根节点信息
    page.getRootBase = async () => {

        if (data.rootNode.id == undefined) {
            data.rootNode.id = ""
        }

        if (data.rootNode.parentId == undefined) {
            data.rootNode.parentId = ""
        }

        if (data.rootNode.id == "" && data.rootNode.parentId == "") {
            return;
        }

        if (!data.rootApi) {
            tg.msg.error(page.$t("ead.page.eciPage.treePage.M00270"));
            //页面没有设置rootApi 【{0}】

            return;
        }
        const request = new TgHttp()
        request.id = data.rootNode.id;
        request.parentId = data.rootNode.parentId;

        let res = await request.build(data.rootApi).post()

        if (res.success) {
            data.rootNode.id = res.result.id;
            data.rootNode.label = res.result.label;
        }
    }

    //不采用懒加载，直接获取全部Tree数据
    page.getAllTreeData = async () => {
        let request = new TgHttp();
        request.id = page.data.rootNode.id;
        request.all = "Y"
        request.parentId = page.data.rootNode.parentId;
        request.searchData = page.data.entity.searchData;
        let response = await request.build(page.data.treeApi).post();
        if (response.success) {
            page.data.treeData = response.result;
        }
    }

    page.loadChildren = async (node, resolve) => {
        // debugger
        // 懒加载先保存当前选中的节点key, 展开后再设置一遍解决展开某选中节点后选中被取消的问题
        if (page.context.getRef("tree") != undefined)
            page.data.checkedKeys = page.context.getRef("tree").getCheckedKeys();
        if (node.level == 0) {
            return resolve([page.data.rootNode]);
        } else {
            let request = new TgHttp();
            request.id = node.data.id;
            let response = await request.build(page.data.treeApi).post();
            let data;
            if (response.success) {
                data = response.result;
                resolve(data);
            }
        }
        if (page.context.getRef("tree") != undefined) {
            page.context.getRef("tree").setCheckedKeys(page.data.checkedKeys);
        }
    }

    //点击树节点
    page.treeClick = (node) => {
        let event = window.event; //|| arguments.callee.caller.arguments[0];

        let me = this;
        page.data.currentNode = node;//当前点击节点
        page.data.parentNode = node.parent;//父节点
        page.data.editNode = node;//当前编辑节点

        if (page.data.editConfig.click || event.ctrlKey) {
            page.treeEditXT(node);
        }
        page.treeEdit(node);
        page.editXTSub(node);

        if (page.data.treeConfig.clickExpand) {
            var isExpand = node.expanded;

            if (!node.expanded) {
                node.expand();
            }
        }

        page.data.clickCount++;

        if (page.data.clickCount >= 2) {
            return;
        }

        this.timer = window.setTimeout(() => {

            if (page.data.clickCount == 1) {
                page.data.clickCount = 0;
            } else if (page.data.clickCount > 1) {
                page.data.clickCount = 0;

                if (isExpand) {
                    page.data.currentNode.expanded = false;
                }
            }

        }, 300);

    }

    //点击树节点
    page.edit = () => {
        let node = page.data.currentNode
        page.data.parentNode = node.parent;//父节点
        page.data.editNode = node;//当前编辑节点
        page.treeEditXT(node);
        page.treeEdit(node);
    }

    page.treeEditXT = (node) => {
        let row = page.getTreeRow(node)
        if (page.editPage) {
            page.editPage.show(row)
        }
    }

    /**编辑 */
    page.treeEdit = (node) => {

    }

    //复制
    page.copy = () => {
        page.data.hasCopy = true;
        page.data.copyNode = page.data.currentNode;
        //已经复制了【{0}】
        tg.msg.success(t("tg.page.tgPage.treePage.M00080", [page.data.copyNode.data.label]));

        page.closeContextMenu();
    }

    //粘贴
    page.paste = async () => {
        page.data.hasCopy = false;
        let node = page.data.copyNode;

        let request = new TgHttp();
        request.parentId = page.data.currentNode.data.id;
        request.copyId = node.data.id;
        let response = await request.build(page.data.copyApi).post();
        if (response.success) {
            tg.msg.success(t("tg.page.tgPage.treePage.M00081"))  //粘贴成功
            //刷新数据
            page.refresh();
        }
        page.closeContextMenu();
    }

    page.addNext = () => {
        let node = page.data.currentNode
        page.data.parentNode = node //父节点
        page.editPage?.showAdd()
        let row = page.getTreeRow(node)
        page.editPage?.setHeadRow(row)
        page.closeContextMenu()

        page.addXTSub();
    }

    page.addSame = (options) => {

        if (options == undefined) {
            options = {};
        }

        let node;

        if (options.byEditPage) {
            node = page.data.editNode;
        } else {
            node = page.data.currentNode;
        }

        page.closeContextMenu()

        if (node.parent.data == undefined) {
            tg.msg.error(t("tg.page.tgPage.treePage.M00082"))//"根节点不可以执行此操作"
            return;
        }

        page.data.parentNode = node.parent

        page.editPage?.showAdd()
        let row = page.getTreeRow(page.data.parentNode)

        page.editPage?.setHeadRow(row)

        page.addXTSub();

    }

    page.editXTSub = (node) => {
        let row = page.getTreeRow(node)
        for (let index in page.context.children) {
            let child = page.context.children[index]
            if (child.sub) {
                page.setHeadRow(row).search();
            }
        }
    }

    page.addXTSub = () => {
        for (let index in page.context.children) {
            let child = page.context.children[index]
            if (page.sub) {
                page.setHeadRow(null).clearTable();
            }
        }
    }

    //删除
    page.del = () => {
        let node = page.data.currentNode;
        if (node.data == undefined) {
            tg.msg.error(t("tg.page.tgPage.treePage.M00083"));
            //删除失败，没有选择要删除的数据
            return;
        }
        if (node.data.id == "") {
            tg.msg.error("//删除失败，删除的节点数据为空");
            //删除失败，删除的节点数据为空
            return;
        }
        if (node.data.id == page.data.rootNode.id) {
            tg.msg.error("//不能删除根节点");
            //不能删除根节点
            return;
        }

        //此操作将删除选中节点下所有子节点，是否确认删除？
        tg.msg.confirm(t("tg.page.tgPage.treePage.M00084")).then(async () => {
            let request = new TgHttp();
            request[page.data.keyField] = node.data.id;
            let response = await request.build(page.data.deleteApi).post()
            if (response.success) {
                const parent = node.parent;
                const children = parent.childNodes;
                const index = children.findIndex(d => d.data.id === node.data.id);
                if (index != -1) {
                    children.splice(index, 1);
                }
                page.deletedXT(node);
                page.deleted(node);
                tg.msg.success(t("tg.page.tgPage.queryPage.M00057"));//删除成功
            }
        });
    }

    /**删除后事件 */
    page.deletedXT = (node) => {
        if (page.editPage && page.data.editNode) {
            if (node.id == page.data.editNode.id) {
                page.editPage?.showAdd()
                page.editPage?.close()
            }
        }
    }

    /**删除后事件 */
    page.deleted = (node) => {

    }

    /**获取选中的节点ID */
    page.getId = () => {
        let checkedKeys = page.context.getRef("tree").getCheckedKeys();
        tg.msg.success(JSON.stringify(checkedKeys));
    }

    //展开
    page.expandAll = () => {
        let selectednode = page.data.currentNode;

        if (selectednode == null || selectednode.data == undefined) {
            selectednode = page.context.getRef("tree").root.childNodes[0];
        }
        //如果只展开到第二层
        selectednode.expanded = true;
        selectednode.expand();
    }

    //折叠
    page.collapseAll = () => {
        page.context.getRef("tree").root.childNodes[0].expanded = false;
    }

    /**修改节点状态 */
    page.changeTreeNodeStatus = (node) => {
        node.expanded = true;
        node.expand();
        for (let i = 0; i < node.childNodes.length; i++) {
            node.childNodes[i].expanded = true;
            node.childNodes[i].expand();
            if (node.childNodes[i].childNodes.length > 0) {
                page.changeTreeNodeStatus(node.childNodes[i]);
            }
        }
    }

    //刷新
    page.refresh = () => {
        page.data.currentNode.loaded = false;
        page.data.currentNode.expand();
        page.closeContextMenu();
    }
    //右击：显示树菜单
    page.showContextMenu = (event, object, node, element) => {
        page.data.currentNode = node;
        page.data.showMenu = true;
        page.context.getRef("tree").setCurrentKey(object.id);

        let el = page.context.getRef("tree").$el;

        let treeBox = $(el).parents(".eci-tree-box")

        if (treeBox.length == 0) {
            //throw Exception(this.$t("ead.page.eciPage.treePage.M00267"));
            //树页面:右键菜单配置不正确,没有包含在容器内
        }

        let $menu = treeBox.find(".eci-tree-menu");

        if ($menu.length == 0) {
            return;
        }

        let left = event.clientX - 10;
        let top = event.clientY - 5;

        let menuHeight = $menu.height();
        let winHeight = this.$(window).height();

        if (top + menuHeight > winHeight) {
            top = winHeight - menuHeight - 15;
        }

        let menu = $menu[0];

        menu.style.cssText = "position: fixed; left: " + left + "px" + "; top: " + top + "px; z-index: 999; cursor:pointer;";

    }
    page.closeContextMenu = () => {
        page.data.showMenu = false;
    }

    /**
     * 拖拽事件
     * @param {*} draggingNode 拖拽节点对应的 Node
     * @param {*} dropNode 结束拖拽时最后进入的节点
     * @param {*} dropType 被拖拽节点的放置位置（before、after、inner）
     * @param {*} ev event
     * @returns
     */
    page.dropAction = async (draggingNode, dropNode, dropType, ev) => {
        if (!page.data.dropApi) {
            page.data.currentNode = dropNode.parent;
            page.refresh();
            return tg.msg.error(t("tg.page.tgPage.treePage.M00085"));//"请配置dropApi地址"
        }
        let id = "";//被拖动当前节点id
        let targetId = "";//拖拽结束时候的节点id
        let point = "";//拖拽节点位置
        if (dropNode.level == 1) {
            tg.msg.error(t("tg.page.tgPage.treePage.M00086"));
            //不能拖到根节点上面
        }
        if (dropType == "inner") {
            point = "append";
        } else if (dropType == "before") {
            point = "top";
        } else if (dropType == "after") {
            point = "bottom";
        }
        id = draggingNode.data.id;
        targetId = dropNode.data.id;
        let request = new TgHttp();
        request.id = id;//当前移动的节点
        request.targetId = targetId;
        request.point = point;
        await request.build(page.data.dropApi).post();
    }

    page.close = () => {
        page.data.visible = false;
    }
    page.clearTree = () => {
        page.context.getRef("tree").data = []
    }
    page.getName = (entity) => {
        return entity[page.data.nameField]
    }
    page.headSave = (entity, isAdd) => {
        if (!page.data.nameField) {
            // '树页面没有配置 nameField 属性'
            return tg.msg.error(t("tg.page.tgPage.treePage.M00087")
            )
        }
        if (isAdd) {
            let item = {
                id: entity[page.data.nameField],
                label: page.getName(entity),
                isLeaf: true,
            }
            let action = function () {
                page.context.getRef("tree").append(data, page.data.parentNode)
                let newNode = page.context.getRef("tree").getNode(data)
                if (page.data.editNode) {
                    page.data.editNode.isCurrent = false
                }
                page.data.editNode = newNode
                page.data.parentNode.isCurrent = false
                page.data.editNode.isCurrent = true
                page.data.currentNode = page.data.editNode
            }
            if (page.data.parentNode.expanded) {
                action()
            } else {
                action()
                page.data.parentNode.expand();
                setTimeout(() => {
                    let id = page.data.editNode.data.id
                    let editNode = page.context.getRef("tree").getNode(id)
                    if (page.data.parentNode) {
                        page.data.parentNode.isCurrent = false
                    }

                    page.data.editNode = editNode
                    page.data.parentNode.isCurrent = false
                    page.data.editNode.isCurrent = true
                    page.data.currentNode = editNode

                }, 200);
            }
        } else {
            let id = page.data.editNode.data.id;
            page.data.editNode = page.context.getRef("tree").getNode(id);
            page.data.editNode.data.label = page.getName(entity);
            if (page.data.parentNode) {
                page.data.parentNode.isCurrent = false;
            }
            page.data.editNode.isCurrent = true;
        }

        page.saved(entity, isAdd);
    }


    page.saved = () => {

    }

    /**获取操作节点对应的数据 */
    page.getTreeRow = (node) => {
        let row = {};

        if (!page.data.keyField) {
            throw Exception("tg.page.tgPage.treePage.M00088")
            // '树页面，没有配置keyField'
        }

        if (!node) {
            throw Exception('tg.page.tgPage.treePage.M00089')//执行getTreeRow方法时,没有传入参数node
        }

        row[page.data.keyField] = node.data.id;

        if (page.data.nameField) {
            row[page.data.nameField] = node.data.label;
        }

        row = tg.extend(node.data, row);

        return row;
    }

    return {
        page
    }

}
