// import { EciHttp } from '@/ead/utils/http/axios/ecihttp'
import {TgHttp} from "../utils/TgRequest/axios/TgHttpBase";
import {EciResponse} from "./ecitrans.d";

export type ErrorMessageMode = "none" | "modal" | "message" | undefined;
export type SuccessMessageMode = ErrorMessageMode;
type Recordable<T = any> = Record<string, T>;

export interface RequestOptions {
    // Splicing request parameters to url
    joinParamsToUrl?: boolean;
    // Format request parameter time
    formatDate?: boolean;
    // Whether to process the request result
    isTransformResponse?: boolean;
    // Whether to return native response headers
    // For example: use this attribute when you need to get the response headers
    isReturnNativeResponse?: boolean;
    // Whether to join url
    joinPrefix?: boolean;
    // Interface address, use the default apiUrl if you leave it blank
    apiUrl?: string;
    // 请求拼接路径
    urlPrefix?: string;
    // Error message prompt type
    errorMessageMode?: ErrorMessageMode;
    // Success message prompt type
    successMessageMode?: SuccessMessageMode;
    // Whether to add a timestamp
    joinTime?: boolean;
    ignoreCancelToken?: boolean;
    // Whether to send token in header
    withToken?: boolean;
    isCore?: false,
    // 请求重试机制
    retryRequest?: RetryRequest;

    //逻辑成功时处理方式
    onSuccess?: (res: EciResponse, options: RequestOptions) => void;
    //逻辑失败时处理方式
    onError?: (res: EciResponse, options: RequestOptions) => void;
    //默认处理逻辑是否执行
    cancelDefaultHandler?: boolean;
    resendFunction?: Function;
    requestData?: any;
    urlParam?: Recordable;
    eciHttp?: TgHttp;
    uploadFiles?: UploadFileParams[];
    file?: false,
    noMsg: false,//报错，是否需要框架来捕获错误信息。默认是要的

}

export interface RetryRequest {
    isOpenRetry: boolean;
    count: number;
    waitTime: number;
}

export interface Result<T = any> {
    code: number;
    msg: string;
    data: T;
}

// multipart/form-data: upload file
export interface UploadFileParams {
    // Other parameters
    data?: Recordable;
    // File parameter interface field name
    name?: string;
    // file name
    file: File | Blob;
    // file name
    filename?: string;

    [key: string]: any;
}
