 import corytoJs from './crypto-js'
 import secretRsa from './secret-rsa'

 let messageEntity = {
   messageHead: {
     messageId: '',
     bizType: '',
     messageType: '',
     signInfo: '',
     keyInfo: '',
     messageDate: '',
     senderId: '',
     receive: '',
     version: ''
   },
   messageBody: {
     data: ''
   }
 }

 let message = {
   data: ''
 }

 const encryptionLevel = '1'

 
 function ReceiveHandle(text) {
  if (encryptionLevel === '0') {
    return text
  }
  if (text === undefined) {
    text = ''
  }
  if (encryptionLevel === '1') {
    return ReceiveHandleLv1(text)
  } else if (encryptionLevel === '2') {
    return ReceiveHandleLv2(text)
  }
}

function ReceiveHandleLv1(text) {
  let dataBase64 = window.atob(text.data);
  let dataEntity = JSON.parse(dataBase64);

  let keyInfo = dataEntity.messageHead.keyInfo;
  let data = dataEntity.messageBody.data;

  let deKey = secretRsa.DecryptRsa(keyInfo);//解密key
  let deData = corytoJs.DecryptAes(data,deKey);//解密内容

  return deData;
 }

 function ReceiveHandleLv2(text) {
  let dataBase64 = window.atob(text.data);
  let dataEntity = JSON.parse(dataBase64);

  let signInfo = dataEntity.messageHead.signInfo;
  let keyInfo = dataEntity.messageHead.keyInfo;
  let data = dataEntity.messageBody.data;

  let deKey = secretRsa.DecryptRsa(keyInfo);//解密key
  let deData = corytoJs.DecryptAes(data,deKey);//解密内容

  let deSign = secretRsa.DecryptRsa(signInfo);
  let enSing = corytoJs.CryptoSHA256(deData);

  if(deSign === enSing){
    return JSON.parse(deData);
  }
 }

 function SendHandle(text) {
   if (encryptionLevel === '0') {
     return text
   }
   if (text === undefined) {
     text = ''
   }
   if (encryptionLevel === '1') {
     return SendHandleLv1(text)
   } else if (encryptionLevel === '2') {
     return SendHandleLv2(text)
   }
 }

 function SendHandleLv1(text) {
   if (text === undefined) {
     text = ''
   }
   let msg = JSON.stringify(text);
   let key = corytoJs.GenerateKey(); //生成key
   let msgData = corytoJs.EncryptAes(msg, key); //生成加密data
   let keyRsa = secretRsa.EncryptRsa(key);

   messageEntity.messageHead.messageId = window.crypto.randomUUID();
   messageEntity.messageHead.signInfo = '';
   messageEntity.messageHead.keyInfo = keyRsa;
   messageEntity.messageBody.data = msgData;
   let str = JSON.stringify(messageEntity);
   message.data = window.btoa(str);
   return message;
 }

 function SendHandleLv2(text) {
   if (text === undefined) {
     text = ''
   }
   let msg = JSON.stringify(text);
   let key = corytoJs.GenerateKey(); //生成key
   let msgData = corytoJs.EncryptAes(msg, key); //生成加密data
   let sign256 = corytoJs.CryptoSHA256(msg); //生成SHA256
   let signRsa = secretRsa.EncryptRsa(sign256); //对SHA256进行RSA加密（sign）
   let keyRsa = secretRsa.EncryptRsa(key);

   messageEntity.messageHead.messageId = window.crypto.randomUUID();
   messageEntity.messageHead.signInfo = signRsa;
   messageEntity.messageHead.keyInfo = keyRsa;
   messageEntity.messageBody.data = msgData;
   let str = JSON.stringify(messageEntity);
   message.data = window.btoa(str);
   return message;
 }

 export default {
   SendHandle,
   ReceiveHandle
 }