const useDraggable = (customDialogWrap?: HTMLElement) => {
	if (customDialogWrap == null) {
		return
	}
	let offsetX: number, offsetY: number

	/**
	 * The function `onDragStart` calculates the offset between the mouse position and the position of an
	 * element.
	 * @param {DragEvent} event - The event parameter is a DragEvent object that represents the dragstart
	 * event. It contains information about the drag operation, such as the position of the mouse cursor
	 * when the drag started.
	 */
	const onDragStart = (event: DragEvent) => {
		offsetX = event.clientX - customDialogWrap.offsetLeft
		offsetY = event.clientY - customDialogWrap.offsetTop
	}

	customDialogWrap.addEventListener('dragstart', onDragStart)

	/**
	 * The onDragOver function prevents the default behavior of a drag event.
	 * @param {DragEvent} event - The event parameter is of type DragEvent, which is an event object that
	 * is triggered when an element is being dragged over another element.
	 */
	const onDragOver = (event: DragEvent) => {
		event.preventDefault()
	}
	document.body.addEventListener('dragover', onDragOver)

	/**
	 * The onDrop function updates the position of a custom dialog box based on the client's mouse
	 * coordinates.
	 * @param {DragEvent} event - The event parameter is a DragEvent object that represents the drag event
	 * that occurred. It contains information about the event, such as the coordinates of the mouse
	 * pointer when the event occurred.
	 */
	const onDrop = (event: DragEvent) => {
		event.preventDefault()
		const x = event.clientX - offsetX
		const y = event.clientY - offsetY
		customDialogWrap.style.left = x + 'px'
		customDialogWrap.style.top = y + 'px'
	}
	document.body.addEventListener('drop', onDrop)
}

export default useDraggable
