import {defineConfig} from "vite";
import vue from '@vitejs/plugin-vue'
import {createSvgIconsPlugin} from 'vite-plugin-svg-icons';

const path = require('path')

export default defineConfig({
    // root: "",//表示项目跟目录
    /**端口号 可改 */
    base: './',
    mode: '',//设置开发模式还是生产模式
    plugins: [
        vue(),
        createSvgIconsPlugin({
            // Specify the icon folder to be cached
            //可以使用数组：iconDirs: ['src/assets/icons', 'src/assets/custom-icons']
            iconDirs: [path.resolve(process.cwd(), 'src/web/extension/svg')],
            // Specify symbolId format
            symbolId: 'icon-[dir]-[name]',
            /**
             * 自定义插入位置
             * @default: body-last
             */
            // inject?: 'body-last' | 'body-first'

            /**
             * custom dom id
             * @default: __svg__icons__dom__
             */
            // customDomId: '__svg__icons__dom__',
            
            /**
			 * 需要注入 index.html ejs 模版的数据
			 */
			inject: {
				tags: [
					{
						injectTo: 'head-prepend',
						tag: 'script',
						attrs: { src: '/lib/luckysheet/plugin.js' }
					},
					{
						injectTo: 'head-prepend',
						tag: 'script',
						attrs: { src: '/lib/luckysheet/luckysheet.umd.js' }
					}
				]
			}
        })
    ],
    resolve: {
        alias: {
            /*
              路径别名
              若为文件系统路径必须是绝对路径的形式，否则将以别名原样呈现，不会解析为文件系统路径路径
            */
            '@': path.resolve(__dirname, './src'),
            // 框架代码文件夹
            '@core': path.resolve(__dirname, './src/tiangongCore'),
            // 框架开发功能
            '@webApp': path.resolve(__dirname, './src/web/app'),
            // 框架页面
            '@appTiangong': path.resolve(__dirname, './src/web/appTiangong'),
            // 公共方法
            '@coreTools': path.resolve(__dirname, './src/tiangongCore/utils/TgMethods'),
            // 请求方法
            '@coreRequest': path.resolve(__dirname, './src/tiangongCore/utils/TgRequest'),
            // 请求方法
            '@coreHttp': path.resolve(__dirname, './src/tiangongCore/utils/TgRequest/TgHttp'),
            // 全局方法
            '@coreTg': path.resolve(__dirname, './src/tiangongCore/utils/TgPublic'),
            // 全局方法 Plus
            '@coreTgPlus': path.resolve(__dirname, './src/tiangongCore/utils/common/tg'),
            // QueryPage模型
            '@coreQueryPage': path.resolve(__dirname, './src/tiangongCore/models/QueryPage/QueryPage'),
            // EditPage模型
            '@coreEditPage': path.resolve(__dirname, './src/tiangongCore/models/EditPage/EditPage'),
            // EditTab模型
            '@coreEditTab': path.resolve(__dirname, './src/tiangongCore/models/EditTab/EditTab'),
            // CommonPage模型
            '@coreCommonPage': path.resolve(__dirname, './src/tiangongCore/models/CommonPage'),
            // componentSetting
            '@componentSetting': path.resolve(__dirname, './src/web/app/setting/componentSetting.js'),
            // 框架组件
            '@coreComponents': path.resolve(__dirname, './src/tiangongCore/components/business'),
            "@i18nHelper": path.resolve(__dirname, './src/tiangongCore/locale/i18nHelper'),
            'vue-i18n': 'vue-i18n/dist/vue-i18n.cjs.js',
            // image
            '@coreImages': path.resolve(__dirname, './src/tiangongCore/assets/images'),
        },
    },// 跨域设置
    css: { // luo. warning: "@charset"
        postcss: {
            plugins: [
                {
                    postcssPlugin: 'internal:charset-removal',
                    AtRule: {
                        charset: (atRule) => {
                            if (atRule.name === 'charset') {
                                atRule.remove();
                            }
                        }
                    }
                }
            ]
        }
    },
    server: {
        open: true,
        public: '0.0.0.0:9528',
        host: "0.0.0.0",  //通过网络IP访问服务
        port: 60001,//指定服务器端口。注意：如果端口已经被使用，Vite 会自动尝试下一个可用的端口，所以这可能不是服务器最终监听的实际端口
        strictPort: false,//配合port,如果端口被占用 设置为true，就会自动退出不会尝试寻找下一个出口
        hmr: true,
        proxy: { //关于代理的配置w
            '^/api/(sysDataHelp|loginByPwd|logout|selectOneByUserId|sysEnterpriseInfo|updateTopic|sysDataHelp)([/.*]?)': {
                // target: 'http://192.168.106.185:3001',
                target: "http://localhost:9527",
                changeOrigin: true,
                rewrite: path => path.replace(/^\/api/, '')
            },
            '/api': {
                // target: 'http://192.168.106.187/LJGW/V100',
                target: "http://localhost:9527",
                changeOrigin: true,
                rewrite: path => path.replace(/^\/api/, '')
            },
        },
    },
    // 忽略导入文件类型的扩展名列表
    // build打包构建的时候需要进行的相关配置
    build: {
        target: "es2020",//关于配置浏览器兼容性，转换过程将会由esbuild执行，如果是自定义目标也可以试一个es版本、一个浏览器版本或者是多个数组[]
        outDir: './dist',//打包输出路径   默认为 dist
        assetsDir: "source",//指定生成静态资源的存放路径
        assetsInlineLimit: 4096,//kb,小于此阈值的导入或引用资源将内联为base64编码，以避免额外的http请求。设置为0 可以完全禁用此项。
        cssCodeSplit: false,//启用/禁用 CSS 代码拆分。当启用时，在异步 chunk 中导入的 CSS 将内联到异步 chunk 本身，并在块加载时插入。
        // 如果禁用，整个项目中的所有 CSS 将被提取到一个 CSS 文件中
        sourcemap: true,//默认为false;构建后是否生成source map文件
        rollupOptions: {//自定义底层的RollUp打包配置，这与Rollup配置文件导出的选项相同，并将与Vite的内部Rollup选项合并
        },
        emptyOutDir: true,//boolean 如果构建输出目录在根目录的话，会抛出一个警告避免删除掉重要文件。
        commonjsOptions: {},//传递给@rollup/plugin-commonjs 插件的选项。
        brotliSize: false,//默认为true 弃用/禁用压缩大小报告，压缩大型输出文件可能会很慢，所以需要禁用该功能从而提高大型项目的构建功能
        chunkSizeWarningLimit: 500,//kb chunk大小警告的限制
        terserOptions: {//相关配置  https://terser.org/docs/api-reference#minify-options
            parse: {},//如果希望指定一些其他 解析选项 则传递一个对象
            compress: {//默认为空对象跳过压缩，或者传递自定义压缩选项
                drop_console: false,//移除console.log
                drop_debugger: true
            }
        }
    },
    //引入第三方配置
    optimizeDeps: {
        include: ["element-plus/lib/locale/lang/zh-cn"],
        esbuildoptions: {
            target: 'es2020'
        }
    },
    "compilerOptions": {
        "types": ["vite/client"]
    },
    // define: {
    //     global: "window"
    // }
})

